@@ -3,22 +3,21 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
- * This file contains Original Code and/or Modifications of Original Code
- * as defined in and that are subject to the Apple Public Source License
- * Version 2.0 (the 'License'). You may not use this file except in
- * compliance with the License. Please obtain a copy of the License at
- * http://www.opensource.apple.com/apsl/ and read it before using this
- * file.
+ * "Portions Copyright (c) 1999 Apple Computer, Inc.  All Rights
+ * Reserved.  This file contains Original Code and/or Modifications of
+ * Original Code as defined in and that are subject to the Apple Public
+ * Source License Version 1.0 (the 'License').  You may not use this file
+ * except in compliance with the License.  Please obtain a copy of the
+ * License at http://www.apple.com/publicsource and read it before using
+ * this file.
  * 
  * The Original Code and all software distributed under the License are
  * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
- * Please see the License for the specific language governing rights and
- * limitations under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
+ * License for the specific language governing rights and limitations
+ * under the License."
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -140,7 +139,7 @@ main(argc, argv)
 	struct stat st;
 	struct timeval tp;
 	struct utmp utmp;
-	int ask, ch, cnt, fflag, hflag, pflag, quietlog, rootlogin = 0, rval;
+	int ask, ch, cnt, oflag = 0, fflag, hflag, pflag, quietlog, rootlogin = 0, rval;
 	uid_t uid;
 	uid_t euid;
 	gid_t egid;
@@ -179,8 +178,11 @@ main(argc, argv)
 
 	fflag = hflag = pflag = 0;
 	uid = getuid();
-	while ((ch = getopt(argc, argv, "fh:p")) != EOF)
+	while ((ch = getopt(argc, argv, "1fh:p")) != EOF)
 		switch (ch) {
+		case '1':
+			oflag = 1;
+			break;
 		case 'f':
 			fflag = 1;
 			break;
@@ -262,7 +264,7 @@ main(argc, argv)
 	} else {
 
 		rval = pam_authenticate(pamh, 0);
-		while( (cnt++ < 10) && ((rval == PAM_AUTH_ERR) ||
+		while( (!oflag) && (cnt++ < 10) && ((rval == PAM_AUTH_ERR) ||
 				(rval == PAM_USER_UNKNOWN) ||
 				(rval == PAM_CRED_INSUFFICIENT) ||
 				(rval == PAM_AUTHINFO_UNAVAIL))) {
@@ -652,6 +654,16 @@ dolastlog(quiet)
 	struct lastlog ll;
 	int fd;
 
+	/* HACK HACK HACK: This is because HFS doesn't support sparse files
+	 * and seeking into the file too far is too slow.  The "solution"
+	 * is to just bail if the seek time for a large uid would be too
+	 * slow.
+	 */
+	if(pwd->pw_uid > 100000) {
+		syslog(LOG_NOTICE, "User login %s (%d) not logged in lastlog.  UID too large.", pwd->pw_name, pwd->pw_uid);
+		return;
+	}
+
 	if ((fd = open(_PATH_LASTLOG, O_RDWR, 0)) >= 0) {
 		(void)lseek(fd, (off_t)pwd->pw_uid * sizeof(ll), L_SET);
 		if (!quiet) {
